@@ -5,10 +5,13 @@ from git_code_debt.file_diff_stat import SpecialFileType
 from git_code_debt.file_diff_stat import Status
 from git_code_debt.metric import Metric
 from git_code_debt.metrics.base import DiffParserBase
+from git_code_debt.metrics.base import MetricInfo
 
 
 class SymlinkCount(DiffParserBase):
-    def get_metrics_from_stat(self, file_diff_stats):
+    """Counts the number of symlinks in the repository."""
+
+    def get_metrics_from_stat(self, _, file_diff_stats):
         symlink_delta = 0
 
         for file_diff_stat in file_diff_stats:
@@ -23,7 +26,8 @@ class SymlinkCount(DiffParserBase):
                 elif file_diff_stat.status is Status.DELETED:
                     symlink_delta -= 1
 
-        yield Metric(type(self).__name__, symlink_delta)
+        if symlink_delta:
+            yield Metric(type(self).__name__, symlink_delta)
 
-    def get_possible_metric_ids(self):
-        return [type(self).__name__]
+    def get_metrics_info(self):
+        return [MetricInfo.from_class(type(self))]
